package com.asgteach.capitalpunishment.views;

import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.asgteach.capitalpunishment.CapitalPunishment;
import com.asgteach.capitalpunishment.model.Question;
import com.asgteach.capitalpunishment.model.Quiz;
import com.asgteach.capitalpunishment.model.WorldCountries;
import com.gluonhq.charm.glisten.control.Avatar;
import com.gluonhq.charm.glisten.control.FloatingActionButton;
import com.gluonhq.charm.glisten.control.ProgressIndicator;
import com.gluonhq.charm.glisten.layout.layer.SidePopupView;
import javafx.animation.Animation;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.beans.binding.Bindings;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.geometry.Pos;
import javafx.geometry.Side;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Toggle;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.ScrollEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.util.Duration;

public class QuizPresenter {

    private static final Double STARTTIME = 10.0;
    private Timeline timeline;
    private final DoubleProperty timeSeconds
            = new SimpleDoubleProperty(STARTTIME);
    private final Quiz quiz = new Quiz();
    private Question question;
    private boolean newQuiz = false;
    private double totalTime = 0.0;
    private static final int BELGIUM = 18;
    private ImageView rightImageView;
    private ImageView wrongImageView;
    private ImageView timeoutImageView;
    private Label feedbackLabel;
    private Label reportLabel;

    @FXML
    private View primary;
    @FXML
    private Label timerLabel;
    @FXML
    private Label instructionLabel;
    @FXML
    private Button nextQuestionButton;
    @FXML
    private Label questionLabel;
    @FXML
    private ToggleGroup selectionToggleGroup;
    @FXML
    private Label remainingQLabel;
    @FXML
    private ProgressIndicator progress;
    @FXML
    private VBox container;
    String saveColor;
    @FXML
    private ImageView globeImageView;
    private FloatingActionButton fb;

    public void initialize() {

        primary.showingProperty().addListener(
                (obs, oldValue, newValue) -> {
                    if (newValue) {
                        AppBar appBar
                        = MobileApplication.getInstance().getAppBar();
                        appBar.setTitleText("Quiz");
                    }
                });
        
        primary.effectiveNodeOrientationProperty().addListener(
                (obs, ov, nv) -> {
                    System.out.println("orientation change");
                });
        // Control show/hide of the floating action button based on 3 things:
        // 1. if the animation status is running, hide it
        // 2. if the timeout popup layer is showing, hide it
        // 3. if the end-of-quiz status layer is showing, hide it
        fb = new FloatingActionButton(
                MaterialDesignIcon.MAP.text, e -> {
                    MobileApplication.getInstance().switchView(
                            CapitalPunishment.SECONDARY_VIEW);
                });
        fb.showOn(primary);
        rightImageView = new ImageView(new Image(
                QuizPresenter.class.getResourceAsStream("check.png")));
        wrongImageView = new ImageView(new Image(
                QuizPresenter.class.getResourceAsStream("delete.png")));

        // Configure the SidePopupView for feedback
        MobileApplication.getInstance().addLayerFactory("Popup", () -> {
            timeoutImageView
                    = new ImageView(new Image(
                            QuizPresenter.class.getResourceAsStream("delete.png")));
            feedbackLabel = new Label();
            final HBox hbox = new HBox(timeoutImageView, feedbackLabel);
            hbox.setAlignment(Pos.CENTER);
            hbox.setSpacing(10);
            hbox.setStyle("-fx-background-color: white; -fx-padding: 10;");
            SidePopupView sidePopupView = new SidePopupView(hbox);
            sidePopupView.setSide(Side.BOTTOM);
            // Hide the floating action button if the timeout layer is showing
            sidePopupView.showingProperty().addListener(
                (obs, oldValue, newValue) -> {
                    if (newValue) {
                        System.out.println("Timeout layer showing");
                        fb.hide();
                    } else {
                        System.out.println("Timeout layer hidden");
                        fb.show();
                    }
                });
            return sidePopupView;
        });
        MobileApplication.getInstance().addLayerFactory("SummaryStats", () -> {
            Avatar avatar = new Avatar(30, new Image(
                    QuizPresenter.class.getResourceAsStream("star.jpg")));
            reportLabel = new Label();
            final HBox hbox = new HBox(avatar, reportLabel);
            hbox.setAlignment(Pos.CENTER_LEFT);
            hbox.setSpacing(10);
            hbox.setStyle("-fx-background-color: #d7e8f7; "
                    + "-fx-padding: 20 5 10 10;");
            SidePopupView reportPopupView = new SidePopupView(hbox);
            reportPopupView.setSide(Side.TOP);
            // Hide the floating action button if the report layer is showing
            reportPopupView.showingProperty().addListener(
                (obs, oldValue, newValue) -> {
                    if (newValue) {
                        System.out.println("Report layer showing");
                        fb.hide();
                    } else {
                        System.out.println("Report layer hidden");
                        fb.show();
                    }
                });
            return reportPopupView;
        });
        // Capital Punishment initializations
        CapitalPunishment.mapCenter
                = WorldCountries.countryList.get(BELGIUM);
        System.out.println("init country = "
                + CapitalPunishment.mapCenter.getCapital()
                + ", latitude = "
                + CapitalPunishment.mapCenter.getLatitude()
                + ", longitude = "
                + CapitalPunishment.mapCenter.getLongitude());
        instructionLabel.textProperty().bind(Bindings.concat(
                "Your score is ",
                quiz.scoreProperty(),
                " out of ",
                quiz.questionsAskedProperty(), "."));
        remainingQLabel.textProperty().bind(
                Bindings.subtract(Quiz.QUIZTOTAL,
                        quiz.questionsAskedProperty()).asString());

        timeSeconds.set(STARTTIME);
        // Bind the timerLabel text property to the timeSeconds property
        timerLabel.textProperty().bind(
                Bindings.format("%4.1f", timeSeconds));
        timerLabel.setPrefWidth(40);
        // Configure the progress
        // Bind the progress progressProperty
        // to the timeSeconds property
        progress.setRadius(23);
        progress.progressProperty().bind(
                timeSeconds.divide(STARTTIME));

        timerLabel.toFront();
        // Initialize the timeline
        timeline = new Timeline();
        timeline.getKeyFrames().add(
                new KeyFrame(Duration.seconds(STARTTIME + 0),
                        new KeyValue(timeSeconds, 0)));

        timeline.setOnFinished((ActionEvent event) -> {
            System.out.println("Timer finished.");
            totalTime = totalTime + STARTTIME - timeSeconds.doubleValue();
            quiz.doWrongAnswer();

            System.out.println("Wrong answer--time out");
            System.out.println("The answer is "
                    + question.getTargetCountry().getCapital()
                    + ".");

            MobileApplication.getInstance().showLayer("Popup");
            feedbackLabel.setText("Time's up! The answer is "
                    + question.getTargetCountry().getCapital()
                    + ".");
        });

        // Bind the disable property for the quiz UI to timer running or not
        nextQuestionButton.disableProperty().bind(
                timeline.statusProperty().isEqualTo(
                        Animation.Status.RUNNING));
        // Hide the floating action button if the animation timer is running
        timeline.statusProperty().addListener(
                (obs, ov, nv) -> {
                    if (nv.equals(Animation.Status.RUNNING)) {
                        fb.hide();
                    } else {
                        fb.show();
                    }
                });

//        fb.showingProperty().bind(
//                timeline.statusProperty().isEqualTo(
//                        Animation.Status.RUNNING));
        selectionToggleGroup.selectedToggleProperty().addListener(
                (ObservableValue<? extends Toggle> ov,
                        Toggle t, Toggle t1) -> {
                    if (selectionToggleGroup.getSelectedToggle() != null) {
                        if (!timeline.getStatus().equals(
                                Animation.Status.RUNNING)) {
                            return;
                        }
                        // process selection
                        timeline.stop();   // no effect if not running
                        totalTime = totalTime
                        + STARTTIME - timeSeconds.intValue();
                        ToggleButton answer = (ToggleButton) t1.getToggleGroup().getSelectedToggle();
                        System.out.println("Selected Button - "
                                + answer.getText());

                        if (answer.getText().equalsIgnoreCase(
                                question.getTargetCountry().getCapital())) {
                            // right answer!
                            quiz.doRightAnswer();
                            answer.setStyle("-fx-background-color: green;");
                            answer.setText("Right! The answer is\n"
                                    + question.getTargetCountry().getCapital()
                                    + ".");
                            answer.setGraphic(rightImageView);
                        } else {
                            // wrong answer!
                            quiz.doWrongAnswer();
                            answer.setStyle("-fx-background-color: red;");
                            String wronganswer = answer.getText();
                            answer.setText("Sorry! The answer is \n"
                                    + question.getTargetCountry().getCapital()
                                    + ", \nnot " + wronganswer + ".");

                            answer.setGraphic(wrongImageView);
                        }
                    }
                });

        quiz.questionsAskedProperty().addListener(
                (ObservableValue<? extends Number> observable,
                        Number oldValue, Number newValue) -> {
                    // are we there yet?
                    if (newValue.intValue() >= Quiz.QUIZTOTAL) {
                        MobileApplication.getInstance().showLayer("SummaryStats");
                        reportLabel.setText(String.format(
                                "Congratulations!\nYou just finished "
                                + "the\nCapital Punishment Quiz!"
                                + "\nYou got "
                                + quiz.getScore() + " out of "
                                + Quiz.QUIZTOTAL
                                + " questions right."
                                + "\nYour average response time was %4.1fs.",
                                (double) totalTime / quiz.getQuestionsAsked()));

                        System.out.println("total time = "
                                + totalTime + " and questions = "
                                + quiz.getQuestionsAsked());
                        System.out.println("Average time is "
                                + String.format("%5.2f",
                                        (double) totalTime / quiz.getQuestionsAsked()));
                        startOver();
                    }
                });

        // Scroll vertically only and don't zoom
        // (manipulate the translateY property)
        container.setOnScroll((ScrollEvent event) -> {
            if (!event.isInertia()) {
                container.setTranslateY(
                        container.getTranslateY() + event.getDeltaY());
            }
            event.consume();
        });

        container.setOnScrollStarted((ScrollEvent event) -> {
            System.out.println("Container: Scroll started event");
            event.consume();
        });

        container.setOnScrollFinished((ScrollEvent event) -> {
            System.out.println("Container: Scroll finished event");
            event.consume();
        });
    }

    private void startOver() {
        nextQuestionButton.setText("Start New Quiz");
        newQuiz = true;
    }

    private void processNextQuestion() {
        if (newQuiz) {
            quiz.resetQuiz();
            totalTime = 0;
            newQuiz = false;
        }
        globeImageView.setVisible(false);
        nextQuestionButton.setText("Get Next Question");
        timeSeconds.set(STARTTIME);
        question = quiz.getNextQuestion();

        questionLabel.setVisible(true);
        questionLabel.setText("What is the capital of "
                + question.getTargetCountry().getName() + "?");
        for (int i = 0;
                i < selectionToggleGroup.getToggles().size(); i++) {
            ToggleButton radio
                    = (ToggleButton) selectionToggleGroup
                            .getToggles().get(i);
            radio.setText(question.getChoices().get(i).getCapital());
            radio.setSelected(false);
            radio.setVisible(true);
            radio.setStyle("-fx-background-color: "
                    + "-primary-swatch-500; ");
            radio.setGraphic(null);
        }
        CapitalPunishment.mapCenter = question.getTargetCountry();
        timeline.playFromStart();

    }

    @FXML
    private void handleNextQuestion(ActionEvent event) {
        System.out.println("You clicked me!");
        processNextQuestion();
    }

    private void handleStartOver(ActionEvent event) {
        newQuiz = true;
        processNextQuestion();
    }

}
