/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.asgteach.people.views;

import com.asgteach.people.model.Employee;
import com.asgteach.people.model.Model;
import com.asgteach.people.service.RemoteService;
import com.gluonhq.charm.glisten.animation.BounceInRightTransition;
import com.gluonhq.charm.glisten.application.MobileApplication;
import static com.gluonhq.charm.glisten.application.MobileApplication.HOME_VIEW;
import com.gluonhq.charm.glisten.control.Alert;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.control.TextField;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.connect.GluonObservableObject;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javax.inject.Inject;

/**
 *
 * @author gail
 */
public class EditPresenter {

    @FXML
    private View edit;

    @Inject
    private Model model;

    @Inject
    private RemoteService remoteService;

    @FXML
    private TextField empnoField;
    @FXML
    private TextField enameField;
    @FXML
    private TextField jobField;
    @FXML
    private TextField mgrField;
    @FXML
    private TextField salField;
    @FXML
    private TextField commField;
    @FXML
    private TextField depnoField;
    @FXML
    private Button cancel;
    @FXML
    private Button submit;

    public void initialize() {
        edit.setShowTransitionFactory(BounceInRightTransition::new);

        edit.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = MobileApplication.getInstance().getAppBar();
                // make the employee number field disabled if we're in edit mode
                empnoField.disableProperty().bind(model.editMode);
                if (model.editMode.get()) {
                    appBar.setTitleText("Update Employee");
                    System.out.println("Editing for " + model.activeEmployee.get());
                    // set fields with data
                    populateFields();
                } else {
                    appBar.setTitleText("Add Employee");
                }
            }
        });
        cancel.setOnAction(e -> close());
        submit.setOnAction(e -> {
            Employee data = model.editMode.get() ? model.activeEmployee.get() : new Employee();
            data.setEname(enameField.getText());
            data.setJob(jobField.getText());
            // Verify user entered numeric data
            try {
                data.setEmpno(Integer.parseInt(empnoField.getText()));
                data.setMgr(Integer.parseInt(mgrField.getText()));
                data.setComm(Integer.parseInt(commField.getText()));
                data.setDeptno(Integer.parseInt(depnoField.getText()));
                data.setSal(Integer.parseInt(salField.getText()));
                
                LocalDate localDate;
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd-MMM-yyyy");
                if (model.editMode.get()) {
                    localDate = LocalDate.parse(data.getHiredate().substring(0, 10));
                    data.setHiredate(localDate.format(formatter));
                } else {
                    // Use today's date if new
                    localDate  = LocalDate.now();
                    data.setHiredate(localDate.format(formatter));
                }
                GluonObservableObject<String> newData;
                String title;
                if (model.editMode.get()) {
                    newData = remoteService.updateEmployee(String.class, data);
                    title = "Employee Updated";
                } else {
                    newData = remoteService.createEmployee(String.class, data);
                    title = "Employee Added";
                }
                newData.initializedProperty().addListener((obs, ov, nv) -> {
                    if (nv) {
                        if (newData.get().contains("ORA")) {
                            Alert alert = new Alert(javafx.scene.control.Alert.AlertType.ERROR);
                            alert.setTitleText("Oracle Database Error");
                            alert.setContentText(String.format("%s\n%s",
                                    "Error:",
                                    newData.get()));
                            alert.showAndWait();
                        } else {
                            Alert alert = new Alert(javafx.scene.control.Alert.AlertType.INFORMATION);
                            alert.setTitleText(title);
                            alert.setContentText(String.format("%s",
                                    data));
                            alert.showAndWait();
                        }
                        close();
                    }
                });

//                // test getEmployee() -- works
//                GluonObservableObject<WorkForce> newData2 = remoteService.getEmployee(WorkForce.class, "7369");
//                newData2.initializedProperty().addListener((obs, ov, nv) -> {
//                    if (nv) {
//                        System.out.println(newData2.get().getItems().get(0));
//                    }
//                });
            } catch (NumberFormatException ex) {
                // Conversion error with one or more of the
                // numeric data entered in the TextField controls.
                // Tell the user to try again
                Alert alert = new Alert(javafx.scene.control.Alert.AlertType.ERROR);
                alert.setTitleText("Number Formatting Error");
                alert.setContentText(String.format("%s\n%s %s",
                        "Please input a valid number.", "Error:",
                        ex.getLocalizedMessage()));
                alert.showAndWait();

            }

        });
    }

    private void close() {
        clearControls();
        model.activeEmployee.set(new Employee());
        MobileApplication.getInstance().switchView(HOME_VIEW);
    }

    private void clearControls() {
        empnoField.setText("");
        enameField.setText("");
        jobField.setText("");
        mgrField.setText("");
        salField.setText("");
        commField.setText("");
        depnoField.setText("");
    }

    private void populateFields() {
        Employee editData = model.activeEmployee.get();
        empnoField.setText(editData.getEmpno() + "");
        enameField.setText(editData.getEname());
        jobField.setText(editData.getJob());
        mgrField.setText(editData.getMgr() + "");
        salField.setText(editData.getSal() + "");
        commField.setText(editData.getComm() + "");
        depnoField.setText(editData.getDeptno() + "");
    }

}
